@@ -41,7 +41,11 @@ def connect(manager, protocol, account):
     reg.LoadManagers()
 
     mgr = reg.GetManager(manager)
-    conn_bus_name, conn_object_path = mgr[CONN_MGR_INTERFACE].RequestConnection(protocol, account)
+    conn_bus_name, conn_object_path = \
+        mgr[CONN_MGR_INTERFACE].RequestConnection(protocol, account)
     return telepathy.client.Connection(conn_bus_name, conn_object_path)
 
+def connection_from_file(path):
+    manager, protocol, account = read_account(path)
+    return connect(manager, protocol, account)
 
