@@ -100,7 +100,13 @@ class Channel(dbus.service.Object):
         such as contact list channels may not be closed.
 
         Possible Errors:
-        Disconnected, NetworkError, NotImplemented
+
+        * Disconnected
+        * NetworkError
+        * NotImplemented - this channel may never be closed, e.g. a contact
+          list
+        * NotAvailable - this channel is not currently in a state where
+          it can be closed, e.g. a non-empty user-defined contact group
         """
         self.Closed()
         self._conn.remove_channel(self)
@@ -253,16 +259,37 @@ class ChannelTypeContactList(Channel):
     on the server. This channel type has no methods because all of the
     functionality it represents is available via the group interface.
 
-    Singleton instances of this channel type should be created by the
-    connection manager at connection time if the list exists on the server, or
-    may be requested by using the appropriate handle.  These handles can be
-    obtained using RequestHandle with a handle type of
-    CONNECTION_HANDLE_TYPE_LIST and one of the following identifiers:
-     subscribe - the group of contacts for whom you wish to receive presence
-     publish - the group of contacts who may receive your presence
-     hide - a group of contacts who are on the publish list but are temporarily disallowed from receiving your presence
-     allow - a group of contacts who may send you messages
-     deny - a group of contacts who may not send you messages
+    There are currently two types of contact list:
+    CONNECTION_HANDLE_TYPE_LIST is a "magic" server-defined list, and
+    CONNECTION_HANDLE_TYPE_USER_CONTACT_GROUP is a user-defined contact group.
+
+    For server-defined lists like the subscribe list, singleton instances
+    of this channel type should be created by the connection manager at
+    connection time if the list exists on the server, or may be requested
+    by using the appropriate handle.  These handles can be obtained using
+    RequestHandle with a handle type of CONNECTION_HANDLE_TYPE_LIST and
+    one of the following identifiers:
+
+    * subscribe - the group of contacts for whom you wish to receive presence
+    * publish - the group of contacts who may receive your presence
+    * hide - a group of contacts who are on the publish list but are temporarily disallowed from receiving your presence
+    * allow - a group of contacts who may send you messages
+    * deny - a group of contacts who may not send you messages
+
+    These contact list channels may not be closed.
+
+    For user-defined contact groups, instances of this channel type should
+    be created by the connection manager at connection time for each group
+    that exists on the server. New, empty groups can be created by calling
+    RequestHandle with a handle type of
+    CONNECTION_HANDLE_TYPE_USER_CONTACT_GROUP and with the name set to the
+    human-readable UTF-8 name of the group.
+
+    User-defined groups may be deleted by closing the channel, but only if
+    the group is already empty. Closing a channel to a non-empty group is
+    not allowed; its members must be set to the empty set first.
+
+    (FIXME: should the user contact groups have a different type?)
     """
     _dbus_interfaces = [CHANNEL_TYPE_CONTACT_LIST]
 
@@ -279,15 +306,24 @@ class ChannelTypeContactList(Channel):
 class ChannelTypeStreamedMedia(Channel):
     """
     A channel that can send and receive streamed media such as audio or video.
-    The negotiation interface is based off the Farsight project's API for
-    negotiating ICE-style connection candidates, but should be applicable
-    to most streaming media systems.
+    Provides a number of methods for listing and requesting new streams, and
+    signals to indicate when streams have been added, removed and changed
+    status.
 
-    When the negotiations are completed, the ReceivedMediaParameters signal is
-    emitted, containing the 'local' media parameters, which contain the SDP
-    information for the local user's media streams, and the 'remote' media
-    parameters which contains the same information for the remote user's
-    streams.
+    In general this should always be used in conjunction with the IceSignalling
+    interface to exchange connection candidates and codec choices with
+    whichever component is responsible for the streams. However, in certain
+    applications where no candidate exchange is necessary (eg the streams are
+    handled by specialised hardware which is controlled directly by the
+    connection manager), the signalling interface can be omitted and this
+    channel type used simply to control the streams.
+
+    This channel type defines two type-specific capability flags (as used in
+    Connection.Interface.Capabilities):
+      CHANNEL_MEDIA_CAPABILITY_AUDIO = 1
+        The handle is capable of using audio streams within a media channel.
+      CHANNEL_MEDIA_CAPABILITY_VIDEO = 2
+        The handle is capable of using video streams within a media channel.
     """
     def __init__(self, connection, handle):
         """
@@ -297,305 +333,201 @@ class ChannelTypeStreamedMedia(Channel):
         connection - the parent Telepathy Connection object
         """
         Channel.__init__(self, connection, CHANNEL_TYPE_STREAMED_MEDIA, handle)
-        self._media_parameters = {}
-
-    @dbus.service.signal(CHANNEL_TYPE_STREAMED_MEDIA, signature='uos')
-
-    def NewMediaSessionHandler(self, member, session_handler, type):
-        """
-        Signal that a session handler object has been created for a member
-        of this channel. The client should then service this session handler
-        object to provide streaming services.
-
-        Parameter:
-        member - member that the MediaSessionHandler is created for
-        session_handler - object path of the MediaSessionHandler object
-        type - string indicating type of session, eg "rtp"
-
-        Possible Errors:
-        Disconnected, InvalidHandle, NotAvailable (if the contact has sent nothing to us on this channel)
-        """
-        pass
 
     @dbus.service.method(CHANNEL_TYPE_STREAMED_MEDIA, in_signature='',
-                                                      out_signature='a(uos)')
-    def GetSessionHandlers(self):
+                                                      out_signature='a(uuuuuu)')
+    def ListStreams(self):
         """
-        Returns all currently active session handlers on this channel
-        as a list of (member, session_handler_path, type).
-        """
-        pass
-
-    @dbus.service.method(CHANNEL_TYPE_STREAMED_MEDIA, in_signature='',
-                                                      out_signature='a(uuuu)')
-    def GetStreams(self):
-        """
-        Returns an array of structs of contact handles, stream identifiers
-        accompanying stream types and the current state of the stream.
-        Stream identifiers are unsigned integers and are unique for 
-        each contact.
+        Returns an array of structs representing the streams currently active
+        within this channel. Each stream is identified by an unsigned integer
+        which is unique for each stream within the channel.
 
         Stream types are identified by the following values:
           MEDIA_STREAM_TYPE_AUDIO = 0
           MEDIA_STREAM_TYPE_VIDEO = 1
-        Stream states are identified by
-          MEDIA_STREAM_STATE_STOPPED =0
-          MEDIA_STREAM_STATE_PLAYING = 1
-          MEDIA_STREAM_STATE_CONNECTING = 2
-          MEDIA_STREAM_STATE_CONNECTED = 3
-        """
-        pass
 
-    @dbus.service.signal(CHANNEL_TYPE_STREAMED_MEDIA, signature='uuu')
-    def StreamStateChanged(self, contact_handle, stream_id, stream_state):
-        """
-        Signal emitted when a memeber's stream's state changes, as defined
-        in GetStreams
+        Stream states are identified by one of the following values:
+          MEDIA_STREAM_STATE_DISCONNECTED = 0
+            The stream is disconnected.
+          MEDIA_STREAM_STATE_CONNECTING = 1
+            The stream is trying to connect.
+          MEDIA_STREAM_STATE_CONNECTED = 2
+            The stream is connected.
 
-        Parameters:
-        stream_id - stream ID as returned by GetStreams
-        stream_state - new stream state
-        """
-        pass
-
-
-class MediaSessionHandler(dbus.service.Object):
-    """
-    A media session handler is an object that handles a number of synchronised
-    media streams.
-    """
-    def __init__(self, bus_name, object_path):
-        dbus.service.Object.__init__(self, bus_name, object_path)
-
-    @dbus.service.method(MEDIA_SESSION_HANDLER, in_signature='',
-                                                out_signature='')
-    def Ready(self):
-        """
-        Inform the connection manager that a client is ready to handle
-        this SessionHandler.
-        """
-        pass;
-
-    @dbus.service.method(MEDIA_SESSION_HANDLER, in_signature='us',
-                                                out_signature='')
-    def Error(self, errno, message):
-        """
-        Inform the connection manager that an error occured in this session.
-        """
-        pass
-
-    @dbus.service.signal(MEDIA_SESSION_HANDLER, signature='ouu')
-    def NewMediaStreamHandler(self, stream_handler, media_type, direction):
-        """
-        Emitted when a new media stream handler has been created for this
-        session.
-
-        Parameters:
-        stream_handler - an object path to a new MediaStreamHandler
-        media_type - enum for type of media that this stream should handle
-          MEDIA_STREAM_TYPE_AUDIO = 0
-          MEDIA_STREAM_TYPE_VIDEO = 1
-        direction - enum for direction of this stream
+        Stream directions are identified by one of the following values:
           MEDIA_STREAM_DIRECTION_NONE = 0
           MEDIA_STREAM_DIRECTION_SEND = 1
           MEDIA_STREAM_DIRECTION_RECEIVE = 2
           MEDIA_STREAM_DIRECTION_BIDIRECTIONAL = 3
+
+        Pending directionality changes are indicated by a bitwise OR of the
+        following flags:
+          MEDIA_STREAM_PENDING_LOCAL_SEND = 1
+            The local user has been asked to send media by the remote user.
+            Call RequestStreamDirection to indicate whether or not this is
+            acceptable.
+          MEDIA_STREAM_PENDING_REMOTE_SEND = 2
+            The remote user has been asked to send media by the local user.
+            The StreamDirectionChanged signal will be emitted when the remote
+            user accepts or rejects this change.
+
+        Returns:
+        an array of structs containing:
+            the stream identifier
+            the contact handle who the stream is with (or 0 if the stream
+                represents more than a single member)
+            the type of the stream
+            the current stream state
+            the current direction of the stream
+            the current pending send flags
         """
         pass
 
-class MediaStreamHandler(dbus.service.Object):
-    """
-    Handles ths lifetime of a media stream. A client should provide
-    information to this handler as and when is it ready.
-    """
-    def __init__(self, bus_name, object_path):
-        dbus.service.Object.__init__(self, bus_name, object_path)
-
-    @dbus.service.method(MEDIA_STREAM_HANDLER, in_signature='a(usuuua{ss})', 
-                                               out_signature='')
-    def Ready(self, codecs):
+    @dbus.service.method(CHANNEL_TYPE_STREAMED_MEDIA, in_signature='uau',
+                                                      out_signature='a(uuuuuu)')
+    def RequestStreams(self, contact_handle, types):
         """
-        Inform the connection manager that a client is ready to handle
-        this StreamHandler. Also provide it with info about all supported
-        codecs.
+        Request that streams be established to exchange the given types of
+        media with the given member. In general this will try and establish a
+        bidirectional stream, but on some protocols it may not be possible to
+        indicate to the peer that you would like to receive media, so a
+        send-only stream will be created initially. In the cases where the
+        stream requires remote agreement (eg you wish to receive media from
+        them), the StreamDirectionChanged signal will be emitted with the
+        MEDIA_STREAM_PENDING_REMOTE_SEND flag set, and the signal emitted again
+        with the flag cleared when the remote end has replied.
 
         Parameters:
-        codecs - as for SupportedCodecs
+        contact_handle - a contact handle with whom to establish the streams
+        types - an array of stream types (as defined in ListStreams)
+
+        Returns:
+        an array of structs (in the same order as the given stream types)
+        containing:
+            the stream identifier
+            the contact handle who the stream is with (or 0 if the stream
+                represents more than a single member)
+            the type of the stream
+            the current stream state
+            the current direction of the stream
+            the current pending send flags
+
+        Possible Errors:
+        InvalidHandle, InvalidArgument (invalid stream type), NotAvailable (if
+        the contact is not able to do this stream type)
         """
         pass
 
-    @dbus.service.method(MEDIA_STREAM_HANDLER, in_signature='us',
-                                                out_signature='')
-    def Error(self, errno, message):
+    @dbus.service.method(CHANNEL_TYPE_STREAMED_MEDIA, in_signature='au',
+                                                      out_signature='')
+    def RemoveStreams(self, streams):
         """
-        Inform the connection manager that an error occured in this stream.
+        Request that the given streams are removed.
 
         Parameters:
-        errno - id of error, one of the following:
+        streams - an array of stream identifiers (as defined in ListStreams)
+
+        Possible Errors:
+        InvalidArgument (unknown stream identifier)
+        """
+        pass
+
+    @dbus.service.method(CHANNEL_TYPE_STREAMED_MEDIA, in_signature='uu',
+                                                      out_signature='')
+    def RequestStreamDirection(self, stream_id, stream_direction):
+        """
+        Request a change in the direction of an existing stream. In particular,
+        this might be useful to stop sending media of a particular type,
+        or inform the peer that you are no longer using media that is being
+        sent to you.
+
+        Depending on the protocol, streams which are no longer sending in
+        either direction should be removed and a StreamRemoved signal emitted.
+        Some direction changes can be enforced locally (for example,
+        BIDIRECTIONAL -> RECEIVE can be achieved by merely stopping sending),
+        others may not be possible on some protocols, and some need agreement
+        from the remote end. In this case, the MEDIA_STREAM_PENDING_REMOTE_SEND
+        flag will be set in the StreamDirectionChanged signal, and the signal
+        emitted again without the flag to indicate the resulting direction when
+        the remote end has accepted or rejected the change.
+
+        Parameters:
+        stream_id - the stream identifier (as defined in ListStreams)
+        stream_direction - the desired stream direction (as defined in
+            ListStreams)
+
+        PossibleErrors:
+        InvalidArgument (unknown stream ID), NotAvailable (the requested
+        direction is not available on this stream)
+        """
+        pass
+
+    @dbus.service.signal(CHANNEL_TYPE_STREAMED_MEDIA, signature='uuu')
+    def StreamAdded(self, stream_id, contact_handle, stream_type):
+        """
+        Emitted when a new stream has been added to this channel.
+
+        Parameters:
+        stream_id - the stream identifier (as defined in ListStreams)
+        contact_handle - the contact handle who the stream is with (or 0 if it
+            represents more than a single member)
+        stream_type - the stream type (as defined in ListStreams)
+        stream_direction - the stream direction (as defined in ListStreams)
+        """
+        pass
+
+    @dbus.service.signal(CHANNEL_TYPE_STREAMED_MEDIA, signature='u')
+    def StreamRemoved(self, stream_id):
+        """
+        Emitted when a stream has been removed from this channel.
+
+        Parameters:
+        stream_id - the stream identifier (as defined in ListStreams)
+        """
+        pass
+
+    @dbus.service.signal(CHANNEL_TYPE_STREAMED_MEDIA, signature='uus')
+    def StreamError(self, stream_id, errno, message):
+        """
+        Emitted when a stream encounters an error.
+
+        Parameters:
+        stream_id - the stream identifier (as defined in ListStreams)
+        errno - a stream error number, one of the following:
           MEDIA_STREAM_ERROR_UNKNOWN = 0
+            An unknown error occured.
           MEDIA_STREAM_ERROR_EOS = 1
-        message - string describing the error
+            The end of the stream was reached.
+        message - a string describing the error (for debugging purposes only)
         """
         pass
 
-
-    @dbus.service.method(MEDIA_STREAM_HANDLER, in_signature='sa(usuussduss)',
-                                               out_signature='')
-    def NewNativeCandidate(self, candidate_id, transports):
+    @dbus.service.signal(CHANNEL_TYPE_STREAMED_MEDIA, signature='uuu')
+    def StreamDirectionChanged(self, stream_id, stream_direction, pending_flags):
         """
-        Inform this MediaStreamHandler that a new native transport candidate
-        has been ascertained.
+        Emitted when the direction or pending flags of a stream are changed. If
+        the MEDIA_STREAM_PENDING_LOCAL_SEND flag is set, the remote user has
+        requested that we begin sending on this stream. RequestStreamDirection
+        should be called to indicate whether or not this change is acceptable.
 
         Parameters:
-        candidate_id - string identifier for this candidate
-        transports - array of transports for this candidate with fields:
-          component number
-          ip (as a string)
-          port
-          enum for base network protocol
-            MEDIA_STREAM_BASE_PROTO_UDP = 0
-            MEDIA_STREAM_BASE_PROTO_TCP = 1
-          string specifying proto subtype (e.g RTP)
-          string specifying proto profile (e.g AVP)
-          our preference value of this transport (double in range 0-1
-          inclusive)
-            1 signals most preferred transport
-          transport type, one of the following:
-            MEDIA_STREAM_TRANSPORT_TYPE_LOCAL = 0
-              a local address
-            MEDIA_STREAM_TRANSPORT_TYPE_DERIVED = 1
-              an external address derived by a method such as STUN
-            MEDIA_STREAM_TRANSPORT_TYPE_RELAY = 2
-              an external stream relay
-          username - string to specify a username if authentication
-                     is required
-          password - string to specify a password if authentication
-                     is required
+        stream_id - the stream identifier (as defined in ListStreams)
+        stream_direction - the new stream direction (as defined in ListStreams)
+        pending_flags - the new pending send flags (as defined in ListStreams)
         """
         pass
 
-    @dbus.service.method(MEDIA_STREAM_HANDLER, in_signature='',
-                                               out_signature='')
-    def NativeCandidatesPrepared(self):
+    @dbus.service.signal(CHANNEL_TYPE_STREAMED_MEDIA, signature='uu')
+    def StreamStateChanged(self, stream_id, stream_state):
         """
-        Informs the connection manager that all possible native candisates
-        have been discovered for the moment.
-        """
-        pass
-
-    @dbus.service.method(MEDIA_STREAM_HANDLER, in_signature='ss',
-                                               out_signature='')
-    def NewActiveCandidatePair(self, native_candidate_id, remote_candidate_id):
-        """
-        Informs the connection manager that a valid candidate pair
-        has been discovered and streaming is in progress.
-        """
-        pass
-
-    @dbus.service.signal(MEDIA_STREAM_HANDLER, signature='ss')
-    def SetActiveCandidatePair(self, native_candidate_id, remote_candidate_id):
-        """
-        Emitted by the connection manager to inform the client that a
-        valid candidate pair has been discovered by the remote end
-        and streaming is in progress.
-        """
-        pass
-
-    @dbus.service.signal(MEDIA_STREAM_HANDLER, signature='a(sa(usuussduss))')
-    def SetRemoteCandidateList(self, remote_candidates):
-        """
-        Signal emitted when the connection manager wishes to inform the
-        client of all the available remote candidates at once.
+        Emitted when a member's stream's state changes.
 
         Parameters:
-        remote_candidates - a list of candidate id and a list of transports
-        as defined in NewNativeCandidate
+        stream_id - the stream identifier (as defined in ListStreams)
+        stream_state - the new stream state (as defined in ListStreams)
         """
         pass
 
-    @dbus.service.signal(MEDIA_STREAM_HANDLER, signature='sa(usuussduss)')
-    def AddRemoteCandidate(self, candidate_id, transports):
-        """
-        Signal emitted when the connection manager wishes to inform the
-        client of a new remote candidate.
-
-        Parameters:
-        candidate_id - string identifier for this candidate
-        transports - array of transports for this candidate with fields,
-                     as defined in NewNativeCandidate
-        """
-        pass
-
-    @dbus.service.signal(MEDIA_STREAM_HANDLER, signature='s')
-    def RemoveRemoteCandidate(self, candidate_id):
-        """
-        Signal emitted when the connection manager wishes to inform the
-        client that the remote end has removed a previously usable
-        candidate.
-
-        Parameters:
-        candidate_id - string identifier for remote candidate to drop
-        """
-        pass
-
-    @dbus.service.method(MEDIA_STREAM_HANDLER, in_signature='u',
-                                               out_signature='')
-    def CodecChoice(self, codec_id):
-        """
-        Inform the connection manager of the current codec choice.
-        """
-        pass
-
-    @dbus.service.method(MEDIA_STREAM_HANDLER, in_signature='u',
-                                               out_signature='')
-    def StreamState(self, state):
-        """
-        Informs the connection manager of the stream's current state
-        State is as specified in ChannelTypeStreamedMedia::GetStreams.
-        """
-        pass
-
-    @dbus.service.method(MEDIA_STREAM_HANDLER, in_signature='a(usuuua{ss})',
-                                                out_signature='')
-    def SupportedCodecs(self, codecs):
-        """
-        Inform the connection manager of the supported codecs for this session.
-        This is called after the connection manager has emitted SetRemoteCodecs
-        to notify what codecs are supported by the peer, and will thus be an
-        intersection of all locally supported codecs (passed to Ready)
-        and those supported by the peer.
-
-        Parameters:
-        codecs - list of codec info structures containing
-            id of codec
-            codec name
-            media type
-            clock rate of codec
-            number of supported channels
-            string key-value pairs for supported optional parameters
-        """
-        pass
-
-    @dbus.service.signal(MEDIA_STREAM_HANDLER, signature='a(usuuua{ss})')
-    def SetRemoteCodecs(self, codecs):
-        """
-        Signal emitted when the connectoin manager wishes to inform the
-        client of the codecs supported by the remote end.
-
-        Parameters:
-        codecs - as for SupportedCodecs
-        """
-        pass
-
-    @dbus.service.signal(MEDIA_STREAM_HANDLER, signature='b')
-    def SetStreamPlaying(self, playing):
-        """
-        Signal emitted when the connection manager wishes to set the
-        stream playing or stopped.
-        """
-        pass
-
-
 
 class ChannelTypeRoomList(Channel):
     """
@@ -829,6 +761,9 @@ class ChannelTypeText(Channel):
         4 - CHANNEL_TEXT_SEND_ERROR_TOO_LONG
             The outgoing message was too long and was rejected by the
             server.
+        5 - CHANNEL_TEXT_SEND_ERROR_NOT_IMPLEMENTED
+            The channel doesn't support sending text messages to requested
+            contact.
 
         Parameters:
         error - one of the above integer errors
@@ -855,7 +790,7 @@ class ChannelTypeText(Channel):
         flags - a bitwise OR of the message flags as defined above
         text - the text of the message
         """
-        self._pending_messages[id] = (timestamp, sender, type, text)
+        self._pending_messages[id] = (timestamp, sender, type, flags, text)
 
     @dbus.service.signal(CHANNEL_TYPE_TEXT, signature='')
     def LostMessage(self):
@@ -983,6 +918,11 @@ class ChannelInterfaceGroup(dbus.service.Interface):
             call GetHandleOwners to find the owners of these handles, which
             should be done if you wish to eg subscribe to the contact's
             presence.
+        512 - CHANNEL_GROUP_FLAG_ONLY_ONE_GROUP
+            Placing a contact in multiple groups of this type is not allowed
+            and will raise NotAvailable (on services where contacts may only
+            be in one user-defined group, user-defined groups will have
+            this flag).
 
         Returns:
         an integer of flags or'd together
@@ -1141,32 +1081,6 @@ class ChannelInterfaceGroup(dbus.service.Interface):
         """
         pass
 
-    @dbus.service.method(CHANNEL_INTERFACE_GROUP, in_signature='au',
-                                                  out_signature='au')
-    def GetHandleOwners(self, handles):
-        """
-        If the CHANNEL_GROUP_FLAG_CHANNEL_SPECIFIC_HANDLES flag is set on
-        the channel, then the handles of the group members are specific
-        to this channel, and are not meaningful in a connection-wide
-        context such as contact lists. This method allows you to find
-        the owner of the handle if it can be discovered in this channel,
-        or 0 if the owner is not available.
-
-        Parameters:
-        handles - a list of integer handles representing members of the
-            channel
-
-        Returns:
-        an array of integer handles representing the owner handle of
-            the the given room members, in the same order, or 0 if the
-            owner is not available
-
-        Possible Errors:
-        Disconnected, NetworkError, InvalidHandle, InvalidArgument (one
-        of the given handles is not a member)
-        """
-        pass
-
     @dbus.service.signal(CHANNEL_INTERFACE_GROUP, signature='sauauauauuu')
     def MembersChanged(self, message, added, removed, local_pending, remote_pending, actor, reason):
         """
@@ -1189,6 +1103,8 @@ class ChannelInterfaceGroup(dbus.service.Interface):
             The change is due to an invitation.
         5 - CHANNEL_GROUP_CHANGE_REASON_BANNED
             The change is due to a kick+ban operation.
+        6 - CHANNEL_GROUP_CHANGE_REASON_ERROR
+            The change is due to an error occurring.
 
         Parameters:
         message - a string message from the server, or blank if not
@@ -1215,6 +1131,9 @@ class ChannelInterfaceGroup(dbus.service.Interface):
 
 class ChannelInterfaceHold(dbus.service.Interface):
     """
+    THIS INTERFACE IS DEPRECATED AND SHOULD NOT BE USED. Hold functionality
+    should be added to the StreamedMedia channel in due course.
+
     Interface for channels where members may put you on hold, or you may put
     members on hold. This usually only makes sense for channels where you are
     streaming media to or from the members. Hold is defined as requesting
